#include <Arduino.h>
#include "Linduino.h"
#include "LT_I2C.h"
#include "LT3964_1.h"
#include <stdint.h>




//**************************************************************************************
// Send out three bytes over I2C transmission lines
// This is useful for sending a single word of data to a single sub address register
// used by the LT3964
int8_t lt3964_i2c_write_3bytes(uint8_t address, uint8_t reg, uint8_t data)
{
  int8_t ret = 0;
  uint8_t BYTE1;

  if (i2c_start() != 0)                                     // Start I2C Tx
    return (1);                                             // If failure occurs during I2C start, return 1 for failure

  address = address | I2C_WRITE;                            // Concatenate first three bits of Tx sequence with device address
  BYTE1 = (address << 1) | I2C_WRITE_BIT;                   // Shift 7 bits to the left and add a write bit to complete first word

  ret |= i2c_write(BYTE1);                                  // Write 7 bit address with W bit
  ret |= i2c_write(reg);                                    // Write sub address to send data to
  ret |= i2c_write(data);                                   // Write data to sub address registers

  i2c_stop();                                               // Stop I2C Tx

  if (ret != 0)                                             // If there was a NACK return 1
    return (1);
  return (0);                                               // If there was an ACK return 0
}



//**************************************************************************************
int16_t lt3964_i2c_write_4bytes(uint8_t address, uint8_t reg, uint16_t data)
{
  int8_t ret = 0;
  uint16_t BYTE1;

  if (i2c_start() != 0)
    return (1);

  address = address | I2C_WRITE_BIT;
  BYTE1 = (address << 1) | I2C_WRITE_BIT;

  ret |= i2c_write(BYTE1);
  ret |= i2c_write(reg);
  ret |= i2c_write(data);

  i2c_stop();

  if (ret != 0)
    return (1);
  return (0);
}

//**************************************************************************************
// Send  out five bytes over I2C transmission lines
// This is useful for sending data to the two PWM sub address registers
// with corresponding data for each reg
int8_t lt3964_i2c_read_bytes(uint8_t address, uint8_t reg, uint8_t *value)
{
  int8_t ret = 0;
  int8_t retval = 0;

  uint8_t add1;
  uint8_t add2;
  uint8_t BYTE1;
  uint8_t BYTE2;

  if (i2c_start() != 0)                                     // Start I2C Tx
    return (-1);                                            // If failure occurs during I2C start, return 1 for failure

  add1 = address | I2C_WRITE;                               //Concatenate first three bits of Tx sequence with device address
  BYTE1 = (add1 << 1) | I2C_WRITE_BIT;                      //Shift 7 bits to the left and add a write bit to complete first word

  ret |= i2c_write(BYTE1);                                  //Write 7 bit address with W bit
  ret |= i2c_write(reg);                                    //Write sub address to send data to

  if (i2c_repeated_start() != 0)                            //Restart I2C Tx with repeated start status
    return (-1);                                            //If failure occurs during I2C start, return 1 for failure

  add2 = address | I2C_READ;                                //Concatenate first three bits of Tx sequence with device address
  BYTE2 = (add2 << 1) | I2C_READ_BIT;                       //Shift 7 bits to the left and add a write bit to complete first word

  ret |= i2c_write(BYTE2);                                  //Write 7 bit address with R bit

  if (ret != 0)
  {
    i2c_stop();                                             //Stop I2C Tx
    return (-1);
  }
  *value = i2c_read(WITH_ACK);                              //Read back data from device and store in value

  i2c_stop();                                               //Stop I2C Tx
  return (retval);                                          //Return success
}


//**************************************************************************************
// Sends  Broadcast Mode Address to all LT3964 connected to the I2C bus
// to check which device has an alert. Function argument #2 (value) returns
// the 4-BIT address of the LT3964 asserting the fault condition
int8_t lt3964_i2c_broadcast_read(uint8_t *value)
{
  *value = 0b00000000;
  int8_t ret = 0;

  if (i2c_start() != 0)                                     // Start I2C Tx
    return (-1);                                             // If failure occurs during I2C start, return 1 for failure

  ret |= i2c_write(0b00011001);          // Shift 7 bits to the left and add a write bit to complete first word

  if (ret != 0)                                             // If slave does not ACK
  {
    i2c_stop();                                             // Stop I2C Tx
    return (-1);
  }

  *value = i2c_read(WITH_NACK);                           // Read from I2C bus with ACK and store value
  i2c_stop();                                               // Stop I2C Tx

  return (0);                                               // Return success
}



//**************************************************************************************
// Sends  Broadcast Write to all LT3964 connected to the I2C bus
// to synchronize PWM dimming cycles amoung synced slave LT3964s
int8_t lt3964_i2c_broadcast_write(void)
{

  int8_t ret = 0;

  if (i2c_start() != 0)                                     // Start I2C Tx
    return (-1);                                             // If failure occurs during I2C start, return 1 for failure

  ret |= i2c_write(0b00011000);          // Shift 7 bits to the left and add a write bit to complete first word

  if (ret != 0)                                             // If slave does not ACK
  {
    i2c_stop();                                             // Stop I2C Tx
    return (-1);
  }
                          // Read from I2C bus with ACK and store value
  i2c_stop();                                               // Stop I2C Tx

  return (0);                                               // Return success
}
